@@ -43,7 +43,7 @@ struct _GgvPostScriptViewPrivate {
 	gint magstep;
 	gboolean pan;
 	gdouble prev_x, prev_y;
-	gchar *tmp_name, *uri;
+	gchar *tmp_name, *uri, *save_path;
 	guint gconf_notify_id;
 
 	GgvMsgWindow *msg_win;
@@ -515,7 +515,7 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 	GnomeVFSFileSize bytes_read = 0;
 	guchar *buffer;
 	GgvPostScriptView *ps_view;
-	gchar *local_file = NULL;
+	gchar *local_file = NULL, *uri_str;
 	FILE *tmpfile;
 
 	g_return_val_if_fail (data != NULL, -1);
@@ -527,7 +527,9 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 
 	uri = gnome_vfs_uri_new(filename);
 	if(gnome_vfs_uri_is_local(uri)) {
-		local_file = g_strdup(gnome_vfs_uri_get_path(uri));
+		uri_str = gnome_vfs_uri_to_string(uri, GNOME_VFS_URI_HIDE_NONE);
+		local_file = gnome_vfs_get_local_path_from_uri(uri_str);
+		g_free(uri_str);
 	}
 	else {
 		/* open uri */
@@ -786,7 +788,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 	gint *active_rows;
-	gchar *doc, *fname;
+	gchar *doc, *fname, *local_path = NULL;
 
 	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
 	if(active_rows[0] == -1) {
@@ -817,11 +819,13 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 
 	if(doc != NULL) {
 		fname = ggv_file_sel_request_uri(_("Select a file to save pages as"),
-										 NULL);
+										 ps_view->priv->save_path);
 		if(fname) {
-			if(ggv_file_readable(fname)) {
+			if((local_path = gnome_vfs_get_local_path_from_uri(fname)) &&
+			   ggv_file_readable(local_path)) {
 				GtkWidget *dlg;
 		
+				g_free(local_path);
 				dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
 											 GTK_DIALOG_MODAL,
 											 GTK_MESSAGE_QUESTION,
@@ -833,6 +837,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 				case GTK_RESPONSE_NO:
 				case GTK_RESPONSE_DELETE_EVENT:
 					gtk_widget_destroy(dlg);
+					g_free(fname);
 					g_free(doc);
 					return;
 				default:
@@ -841,7 +846,9 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 				gtk_widget_destroy(dlg);
 			}
 			ps_view_save_doc(ps_view, fname, doc, strlen(doc));
-			g_free(fname);
+			if(ps_view->priv->save_path)
+				g_free(ps_view->priv->save_path);
+			ps_view->priv->save_path = fname;
 		}
 		g_free(doc);
 	}
@@ -1673,6 +1680,11 @@ ggv_postscript_view_finalize(GObject *object)
 
 	ps_view_clean_tmp_file(ps_view);
 
+	if(ps_view->priv->save_path) {
+		g_free(ps_view->priv->save_path);
+		ps_view->priv->save_path = NULL;
+	}
+
 	g_free(ps_view->priv);
 
 	G_OBJECT_CLASS(ggv_postscript_view_parent_class)->finalize(object);
