@@ -32,6 +32,8 @@ struct _GgvSidebarPrivate {
 	GtkWidget *checklist;
 	GtkWidget *toggle_all, *toggle_even, *toggle_odd, *clear_all;
 	GtkWidget *coordinates;
+
+	GtkTooltips *toggle_tips;
 };
 
 struct _GgvSidebarClassPrivate {
@@ -64,6 +66,9 @@ ggv_sidebar_finalize (GObject *object)
 
 	sidebar = GGV_SIDEBAR (object);
 
+	if(sidebar->priv->toggle_tips)
+		g_object_unref (G_OBJECT(sidebar->priv->toggle_tips));
+
 	g_free (sidebar->priv);
 
 	G_OBJECT_CLASS (ggv_sidebar_parent_class)->finalize (object);
@@ -277,6 +282,10 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	/* with a sidebar on the left */
 	sidebar->priv->root = gtk_vbox_new(FALSE, 0);
 
+	sidebar->priv->toggle_tips = gtk_tooltips_new();
+	g_object_ref(G_OBJECT(sidebar->priv->toggle_tips));
+	gtk_object_sink(GTK_OBJECT(sidebar->priv->toggle_tips));
+
 	sidebar->priv->coordinates = gtk_label_new(" ");
 	gtk_widget_show(sidebar->priv->coordinates);
 	gtk_box_pack_start(GTK_BOX(sidebar->priv->root),
@@ -291,6 +300,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_widget_show(image);
 	sidebar->priv->toggle_all = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_all), GTK_RELIEF_NONE);
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_all,
+						 _("Toggle marked state of all pages"),
+						 _("Toggle marked state of all pages: previously "
+						   "marked pages will be unmarked and umarked ones "
+						   "will become marked."));
 	gtk_widget_show(sidebar->priv->toggle_all);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_all, FALSE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(sidebar->priv->toggle_all), image);
@@ -299,6 +313,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleodd.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->toggle_odd = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_odd,
+						 _("Toggle marked state of odd pages"),
+						 _("Toggle marked state of odd pages: previously "
+						   "marked odd pages will be unmarked and umarked "
+						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_odd), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_odd);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_odd, FALSE, TRUE, 0);
@@ -308,6 +327,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleeven.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->toggle_even = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_even,
+						 _("Toggle marked state of even pages"),
+						 _("Toggle marked state of even pages: previously "
+						   "marked even pages will be unmarked and umarked "
+						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_even), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_even);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_even, FALSE, TRUE, 0);
@@ -317,6 +341,10 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/clearall.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->clear_all = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->clear_all,
+						 _("Clear marked state of all pages"),
+						 _("Clear marked state of all pages: all pages will "
+						   "be unmarked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->clear_all), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->clear_all);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->clear_all, FALSE, TRUE, 0);
