@@ -14,6 +14,10 @@
 #include <libgnomeui/gnome-window-icon.h>
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <X11/Xlib.h>
+#include <X11/Xatom.h> 
+#include <gdk/gdkx.h>
 
 #include <math.h>
 #include <ctype.h>
@@ -36,58 +40,6 @@ enum {
         TARGET_URI_LIST,
 };
 
-static void
-ggv_window_drag_data_received(GtkWidget *widget,
-                              GdkDragContext *context,
-                              gint x, gint y,
-                              GtkSelectionData *selection_data,
-                              guint info, guint time)
-{
-        GgvWindow *win = GGV_WINDOW(widget);
-
-	if (info != TARGET_URI_LIST)
-		return;
-
-        win->uris_to_open = g_strsplit(selection_data->data, "\r\n", 0);
-        if (context->suggested_action == GDK_ACTION_ASK) {
-                GtkWidget *menu = gtk_menu_new ();
-		
-                bonobo_window_add_popup (BONOBO_WINDOW (win), 
-                                         GTK_MENU (menu), 
-                                         "/popups/DnD");
-                gtk_menu_popup (GTK_MENU (menu),
-                                NULL,
-                                NULL,
-                                NULL,
-                                NULL,
-                                0,
-                                GDK_CURRENT_TIME);
-        }
-        else {
-                GtkWidget *newwin;
-                gchar **uri = win->uris_to_open;
-
-                if(win->current_page < 0)
-                        newwin = GTK_WIDGET(win);
-                else
-                        newwin = NULL;
-                while(*uri && **uri != '\0') {
-#ifdef _DEBUG
-                        g_message("URI %s\n", *uri);
-#endif /* _DEBUG */
-                        if(newwin == NULL) {
-                                newwin = ggv_window_new();
-                                gtk_widget_show(newwin);
-                        }
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri));
-                        newwin = NULL;
-                        uri++;
-                }
-                g_strfreev(win->uris_to_open);
-                win->uris_to_open = NULL;
-        }
-}
-
 static void
 sync_settings_menu_items(GgvWindow *win)
 {
@@ -169,6 +121,187 @@ sync_toolbar_style(GgvWindow *win)
                                              tbstyle, NULL);
 }
 
+static void
+sync_fullscreen_items(GgvWindow *win)
+{
+        if(win->uic != NULL) {
+                bonobo_ui_component_set_prop(win->uic, "/commands/ViewFullscreen", "state",
+                                             win->fullscreen?"1":"0", NULL);
+        }
+        if(win->popup_uic != NULL) {
+                bonobo_ui_component_set_prop(win->popup_uic, "/commands/ViewFullscreen", "state",
+                                             win->fullscreen?"1":"0", NULL);
+        }
+}
+
+/* utility functions to raise window above all others! */
+#define XA_WIN_LAYER         "_WIN_LAYER"
+typedef enum {
+        WIN_LAYER_NORMAL = 4,
+        WIN_LAYER_ABOVE_DOCK = 10
+} GgvWindowLayer;
+static Atom A_XA_WIN_LAYER;
+static gboolean atom_initialized = FALSE;
+
+static void
+ggv_init_atoms()
+{
+        if(!GDK_DISPLAY() || atom_initialized) return;
+        A_XA_WIN_LAYER = XInternAtom(GDK_DISPLAY(), XA_WIN_LAYER, False);
+        atom_initialized = TRUE;
+}
+
+static void
+ggv_window_set_layer(GgvWindow *win, GgvWindowLayer layer)
+{
+        if(!atom_initialized)
+                ggv_init_atoms();
+        if(GTK_WIDGET_MAPPED(win)) {
+                XEvent e;
+                e.type = ClientMessage;
+                e.xclient.type = ClientMessage;
+                e.xclient.window = GDK_WINDOW_XID(GTK_WIDGET(win)->window);
+                e.xclient.message_type = A_XA_WIN_LAYER;
+                e.xclient.format = 32;
+                e.xclient.data.l[0] = (long)layer;
+                e.xclient.data.l[1] = GDK_CURRENT_TIME;
+
+                XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
+                           SubstructureNotifyMask, (XEvent *)&e);
+        }
+        else {
+                long data[1];
+
+                data[0] = layer;
+                XChangeProperty(GDK_DISPLAY(),
+                                GDK_WINDOW_XID(GTK_WIDGET(win)->window),
+                                A_XA_WIN_LAYER, XA_CARDINAL, 32,
+                                PropModeReplace, (unsigned char *)data, 1);
+        }
+}
+
+static void
+ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
+{
+        if(win->fullscreen == fs)
+                return;
+
+        win->fullscreen = fs;
+        if(win->fullscreen) {
+                gint clx, cly, rx, ry, w, h;
+                gdk_window_get_origin(GTK_WIDGET(win)->window, &rx, &ry);
+                gdk_window_get_geometry(GTK_WIDGET(win)->window,
+                                        &clx, &cly, &w, &h, NULL);
+                win->orig_x = rx - clx;
+                win->orig_y = ry - cly;
+                win->orig_width = w;
+                win->orig_height = h;
+                if(win->show_menus) {
+                        bonobo_ui_component_set_prop(win->uic, "/menu",
+                                                     "hidden", "1", NULL);
+                }
+                if(win->show_toolbar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Toolbar",
+                                                     "hidden", "1", NULL);
+                }
+                if(win->show_statusbar) {
+                        gtk_widget_hide(win->statusbar);
+                }
+                if(win->show_sidebar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Sidebar",
+                                                     "hidden", "1", NULL);
+                }
+                win->orig_sm = win->show_menus;
+                win->orig_st = win->show_toolbar;
+                win->orig_ss = win->show_sidebar;
+                win->orig_sss = win->show_statusbar;
+                win->show_menus = win->show_toolbar = win->show_sidebar = win->show_statusbar = FALSE;
+                gtk_window_move(GTK_WINDOW(win), -clx, -cly);
+                gtk_window_resize(GTK_WINDOW(win),
+                                  gdk_screen_width(),
+                                  gdk_screen_height());
+        }
+        else {
+                win->show_menus = win->orig_sm;
+                win->show_toolbar = win->orig_st;
+                win->show_sidebar = win->orig_ss;
+                win->show_statusbar = win->orig_sss;
+                if(win->show_menus) {
+                        bonobo_ui_component_set_prop(win->uic, "/menu",
+                                                     "hidden", "0", NULL);
+                }
+                if(win->show_toolbar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Toolbar",
+                                                     "hidden", "0", NULL);
+                }
+                if(win->show_statusbar) {
+                        gtk_widget_show(win->statusbar);
+                }
+                if(win->show_sidebar) {
+                        bonobo_ui_component_set_prop(win->uic, "/Sidebar",
+                                                     "hidden", "0", NULL);
+                }
+                gtk_window_move(GTK_WINDOW(win), win->orig_x, win->orig_y);
+                gtk_window_resize(GTK_WINDOW(win), win->orig_width, win->orig_height);
+        }
+        ggv_window_set_layer(win, fs?WIN_LAYER_ABOVE_DOCK:WIN_LAYER_NORMAL);
+        sync_settings_menu_items(win);
+        sync_settings_popup_items(win);
+        sync_fullscreen_items(win);
+}
+
+static void
+ggv_window_drag_data_received(GtkWidget *widget,
+                              GdkDragContext *context,
+                              gint x, gint y,
+                              GtkSelectionData *selection_data,
+                              guint info, guint time)
+{
+        GgvWindow *win = GGV_WINDOW(widget);
+
+	if (info != TARGET_URI_LIST)
+		return;
+
+        win->uris_to_open = g_strsplit(selection_data->data, "\r\n", 0);
+        if (context->suggested_action == GDK_ACTION_ASK) {
+                GtkWidget *menu = gtk_menu_new ();
+		
+                bonobo_window_add_popup (BONOBO_WINDOW (win), 
+                                         GTK_MENU (menu), 
+                                         "/popups/DnD");
+                gtk_menu_popup (GTK_MENU (menu),
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
+                                0,
+                                GDK_CURRENT_TIME);
+        }
+        else {
+                GtkWidget *newwin;
+                gchar **uri = win->uris_to_open;
+
+                if(win->current_page < 0)
+                        newwin = GTK_WIDGET(win);
+                else
+                        newwin = NULL;
+                while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                        g_message("URI %s\n", *uri);
+#endif /* _DEBUG */
+                        if(newwin == NULL) {
+                                newwin = ggv_window_new();
+                                gtk_widget_show(newwin);
+                        }
+                        ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                        newwin = NULL;
+                        uri++;
+                }
+                g_strfreev(win->uris_to_open);
+                win->uris_to_open = NULL;
+        }
+}
+
 static void
 control_property_changed_handler(BonoboListener    *listener,
                                  char              *event_name, 
@@ -231,7 +364,7 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
         gchar *fname, *escaped_fname;
 
         fname = ggv_file_sel_request_uri(_("Select a PostScript document"),
-                                         NULL);
+                                         win->filename);
         if(fname) {
                 if(!ggv_window_load(win, fname)) {
                         GtkWidget *dlg;
@@ -285,18 +418,48 @@ verb_FileNew(BonoboUIComponent *uic, gpointer data, const char *cname)
         gtk_widget_show(win);
 }
 
+void
+ggv_window_destroy_all()
+{
+        while(window_list)
+                gtk_widget_destroy(GTK_WIDGET(window_list->data));
+}
+
 static void
 verb_FileExit(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GList *l;
 
-        for(l = window_list; l != NULL; l = l->next)
-                gtk_widget_destroy(GTK_WIDGET(l->data));
-        g_list_free(window_list);
-        window_list = NULL;
+        ggv_window_destroy_all();
         bonobo_main_quit();
 }
 
+static void
+listener_ViewFullscreen (BonoboUIComponent *uic, const char *path,
+                         Bonobo_UIComponent_EventType type, const char *state,
+                         gpointer user_data)
+{
+	GgvWindow *window;
+        gboolean state_f;
+
+	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(GGV_IS_WINDOW (user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state)
+		return;
+
+        window = GGV_WINDOW(user_data);
+        state_f = atoi(state);
+
+        if(!strcmp(path, "ViewFullscreen")) {
+                if(window->fullscreen != state_f)
+                        ggv_window_set_fullscreen(window, state_f);
+        }
+}
+
 static void
 listener_SettingsShow (BonoboUIComponent *uic, const char *path,
                        Bonobo_UIComponent_EventType type, const char *state,
@@ -530,6 +693,8 @@ ggv_window_destroy(GtkObject *object)
                 win->filename = NULL;
         }
 
+        window_list = g_list_remove(window_list, win);
+
         ggv_get_window_size(GTK_WIDGET(object),
                             &ggv_default_width, &ggv_default_height);
 
@@ -735,6 +900,8 @@ ggv_window_new(void)
                                          listener_SettingsShow, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowStatusbar",
                                          listener_SettingsShow, win);
+	bonobo_ui_component_add_listener(uic, "ViewFullscreen",
+                                         listener_ViewFullscreen, win);
         sync_settings_menu_items(win);
 
         if(!win->show_toolbar)
@@ -794,6 +961,8 @@ ggv_window_new(void)
                                                  listener_SettingsShow, win);
                 bonobo_ui_component_add_listener(popup_uic, "SettingsShowStatusbar",
                                                  listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "ViewFullscreen",
+                                                 listener_ViewFullscreen, win);
                 win->popup_uic = popup_uic;
                 sync_settings_popup_items(win);
         }
@@ -862,8 +1031,6 @@ ggv_window_close(GgvWindow *win)
         g_return_if_fail(win != NULL);
         g_return_if_fail(GGV_IS_WINDOW(win));
 
-        window_list = g_list_remove(window_list, win);
-
         gtk_widget_destroy(GTK_WIDGET(win));
 
         if(!window_list)
