@@ -31,7 +31,7 @@ gint ggv_get_index_of_string(gchar *string, gchar **strings);
 gchar *ggv_quote_filename (const gchar *str);
 
 /* escape filename to conform to URI specification */
-gchar *ggv_escape_filename(const gchar *fname);
+gchar *ggv_filename_to_uri(const gchar *fname);
 
 /* If file exists and is a regular file then return its length, else -1 */
 gint ggv_file_length (const gchar *filename);
