@@ -62,6 +62,47 @@ gtk_check_list_get_type (void)
         return check_list_type;
 }
 
+static gboolean
+key_press_event(GtkWidget *w, GdkEventKey *e)
+{
+        GtkCheckList *cl = GTK_CHECK_LIST(w);
+        GtkTreeSelection *sel;
+        GtkTreeIter iter;
+        GtkTreeModel *model;
+        GtkTreePath *path;
+
+        if(e->state != 0 || e->type != GDK_KEY_PRESS) {
+                if(GTK_WIDGET_CLASS(parent_class)->key_press_event)
+                        (*GTK_WIDGET_CLASS(parent_class)->key_press_event)(w, e);
+                return FALSE;
+        }
+
+        sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(cl));
+        if(gtk_tree_selection_get_selected(sel, &model, &iter))
+                path = gtk_tree_model_get_path(model, &iter);
+
+        switch(e->keyval) {
+        case GDK_KP_Page_Up:
+        case GDK_Page_Up:
+                if(gtk_tree_path_prev(path)) {
+                        gtk_tree_view_set_cursor(GTK_TREE_VIEW(cl), path, NULL, FALSE);
+                }
+                gtk_tree_path_free(path);
+                return TRUE;
+        case GDK_KP_Page_Down:
+        case GDK_Page_Down:
+                gtk_tree_path_next(path);
+                gtk_tree_view_set_cursor(GTK_TREE_VIEW(cl), path, NULL, FALSE);
+                gtk_tree_path_free(path);
+                return TRUE;
+        default:
+                gtk_tree_path_free(path);
+                if(GTK_WIDGET_CLASS(parent_class)->key_press_event)
+                        (*GTK_WIDGET_CLASS(parent_class)->key_press_event)(w, e);
+                return FALSE;
+        }
+}
+
 static void
 gtk_check_list_init(GtkCheckList *cl)
 {
@@ -70,6 +111,11 @@ gtk_check_list_init(GtkCheckList *cl)
 static void
 gtk_check_list_class_init (GtkCheckListClass *klass)
 {
+        GtkWidgetClass *wklass;
+
+        wklass = GTK_WIDGET_CLASS(klass);
+        wklass->key_press_event = key_press_event;
+
         parent_class = g_type_class_peek_parent(klass);
 }
 
