@@ -22,6 +22,7 @@
 
 #include "ggv-window.h"
 #include "ggv-prefs.h"
+#include "ggvutils.h"
 
 #define BONOBO_DEBUG
 
@@ -33,10 +34,7 @@ static gchar *filename_stdin;
 static void
 sig_handler(int signum)
 {
-        const GList *l;
-
-        for(l = ggv_get_window_list(); l != NULL; l = l->next)
-                gtk_widget_destroy(GTK_WIDGET(l->data));
+        ggv_window_destroy_all();
         bonobo_main_quit();
 }
 
@@ -45,6 +43,11 @@ create_window(const gchar *filename)
 {
         GgvWindow *win = GGV_WINDOW(ggv_window_new());
 
+        if(win == NULL) {
+                g_warning("Failed to create a new window.");
+                return NULL;
+        }
+
         if(filename)
                 ggv_window_load(win, filename);
 
@@ -98,6 +101,11 @@ create_windows_from_cmdline(gpointer data)
                 win_count--;
         }
 
+        if(ggv_get_window_list() == NULL) {
+                g_warning("No windows could be opened. Exiting...");
+                bonobo_main_quit();
+        }
+
         return FALSE;
 }
 
@@ -125,7 +133,7 @@ save_session(GnomeClient        *client,
 	argv[0] = (gchar *)client_data;
         argc = 1;
         node = ggv_get_window_list();
-        while(node) {
+        while(node && argc < 128) {
                 win = GGV_WINDOW(node->data);
                 if(win->filename) {
                         if(!filename_stdin ||
@@ -138,7 +146,7 @@ save_session(GnomeClient        *client,
                         wnc++;
                 node = node->next;
         }
-	if(wnc > 0) {
+	if(wnc > 0 && argc < 128) {
                 win_opt = g_strdup_printf("--windows=%d", wnc);
                 argv[argc++] = win_opt;
         }
